<?php
declare(strict_types=1);

namespace Inti\Container\Factory;

use Inti\Container\Exception;
use Laminas\ServiceManager\PluginManagerInterface;
use Psr\Container\ContainerInterface;
use ReflectionClass;

use function class_exists;

/**
 * Class PluginManagerFactory
 *
 * @package Inti\Container\Factory
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-container
 */
final class PluginManagerFactory
{
    public function __invoke(ContainerInterface $container, string $requestedName): PluginManagerInterface
    {
        if (!class_exists($requestedName)) {
            throw Exception\InvalidPluginManagerException::classNotFound($requestedName);
        }

        /** @noinspection PhpUnhandledExceptionInspection */
        $refl = new ReflectionClass($requestedName);

        if (!$refl->implementsInterface(PluginManagerInterface::class)) {
            throw Exception\InvalidPluginManagerException::isNotAPluginManager($requestedName);
        }

        $config = $container->has('config') ? $container->get('config') : [];

        /** @noinspection PhpIncompatibleReturnTypeInspection */
        /** @noinspection PhpUnhandledExceptionInspection */
        return $refl->newInstance($container, $config[$requestedName] ?? []);
    }
}
