<?php
declare(strict_types=1);

namespace IntiTest\Container\Factory;

use Inti\Container\Exception\InvalidPluginManagerException;
use Inti\Container\Factory\PluginManagerFactory;
use Inti\Test\TestCase\ContainerTestCase;
use Laminas\Hydrator\HydratorPluginManager;
use Laminas\Hydrator\ReflectionHydrator;
use Laminas\ServiceManager\PluginManagerInterface;
use PHPUnit\Framework\TestCase;

/**
 * Class PluginManagerFactoryTest
 *
 * @package IntiTest\Container\Factory
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-container
 */
final class PluginManagerFactoryTest extends TestCase
{
    use ContainerTestCase;

    private PluginManagerFactory $factory;

    protected function setUp(): void
    {
        $this->setUpContainer();
        $this->factory = new PluginManagerFactory();
    }

    public function testWillThrowExceptionIfPluginManagerDoesntExist()
    {
        $container = $this->createContainer();

        $this->expectException(InvalidPluginManagerException::class);
        ($this->factory)($container, 'invalid\class');
    }

    public function testWillThrowExceptionIfPluginManagerIsOfWrongType()
    {
        $container = $this->createContainer();

        $this->expectException(InvalidPluginManagerException::class);
        ($this->factory)($container, self::class);
    }

    public function testWillCreateConfiguredPluginManagers()
    {
        $container = $this->createContainer([
            HydratorPluginManager::class => [
                'aliases' => [
                    'myalias' => ReflectionHydrator::class,
                ],
            ],
        ]);

        $manager = ($this->factory)($container, HydratorPluginManager::class);

        $this->assertInstanceOf(PluginManagerInterface::class, $manager);
        $this->assertTrue($manager->has('myalias'));
    }
}
